# Generated by Django 4.0.7 on 2022-08-30 16:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("package_monitor", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="distribution",
            name="is_editable",
            field=models.BooleanField(
                default=None,
                help_text="Package has been installed in editable mode, i.e. pip install -e",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="apps",
            field=models.JSONField(
                default=list,
                help_text="List of installed Django apps included in this package",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="has_installed_apps",
            field=models.BooleanField(
                default=False,
                help_text="Whether this package has installed Django apps",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="is_outdated",
            field=models.BooleanField(
                default=None,
                help_text="A package is outdated when there is a newer stable version available",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="used_by",
            field=models.JSONField(
                default=list,
                help_text="List of other distribution packages using this package",
            ),
        ),
    ]
