# Generated by Django 4.0.7 on 2022-09-12 15:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("package_monitor", "0002_add_editable_and_refactor_json_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="distribution",
            name="latest_notified_version",
            field=models.CharField(
                default="",
                help_text="Latest version that has been notified",
                max_length=64,
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="latest_version",
            field=models.CharField(
                default="",
                help_text="Latest version available for this package",
                max_length=64,
            ),
        ),
    ]
